import openai
from time import sleep
import json
import pkg_resources
from .helper import get_secret

class OpenAI:
    def __init__(self):
        # load the connection string from a config file
        config = json.loads(pkg_resources.resource_string(__name__, "config.json"))
        
        self.instance = openai;
        self.instance.api_key = get_secret("openai-api-key")

        self.prompt_limit = config["openai"]["prompt_limit"]
        # What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic.
        self.temperature = config["openai"]["temperature"]
        # An alternative to sampling with temperature, called nucleus sampling, where the model considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered. We generally recommend altering this or temperature but not both.
        self.top_p = config["openai"]["top_p"]
        self.max_tokens = config["openai"]["max_tokens"]
        self.frequency_penalty = config["openai"]["frequency_penalty"]
        self.presence_penalty = config["openai"]["presence_penalty"]
        self.embed_model = config["openai"]["embed_model"]
        self.llmodel = config["openai"]["llmodel"]
    
    def create_embeddings(self, texts):
        responses = self.instance.Embedding.create(input=texts, engine=self.embed_model)
        embeds = [record['embedding'] for record in responses['data']]
        return embeds
    


from azure.storage.blob import BlobServiceClient, ContainerClient
from azure.core.exceptions import ResourceNotFoundError
from .helper import get_secret

class AzureStorage:
    def __init__(self):
        azure_storage_connection_string = get_secret("azure-storage-connection-string")

        self.blob_service_client = BlobServiceClient.from_connection_string(
            azure_storage_connection_string
        )

    def create_container(self, container_name):
        container_client = self.blob_service_client.get_container_client(container_name)
        container_client.create_container()

    def delete_container(self, container_name):
        container_client = self.blob_service_client.get_container_client(container_name)
        container_client.delete_container()

    def list_containers(self):
        containers = self.blob_service_client.list_containers()
        return [container.name for container in containers]

    def update_container_metadata(self, container_name, metadata):
        container_client = self.blob_service_client.get_container_client(container_name)
        container_properties = container_client.get_container_properties()
        container_properties.metadata.update(metadata)
        container_client.set_container_metadata(container_properties.metadata)

    def read_container_metadata(self, container_name):
        container_client = self.blob_service_client.get_container_client(container_name)
        container_properties = container_client.get_container_properties()
        return container_properties.metadata

    def list_blobs(self, container_name):
        container_client = self.blob_service_client.get_container_client(container_name)
        blob_list = container_client.list_blobs()
        return [blob.name for blob in blob_list]

    def upload_blob(self, container_name, blob_name, data, overwrite=False):
        container_client = self.blob_service_client.get_container_client(container_name)
        blob_client = container_client.get_blob_client(blob_name)
        blob_client.upload_blob(data, overwrite=overwrite)

    def download_blob(self, container_name, blob_name):
        container_client = self.blob_service_client.get_container_client(container_name)
        blob_client = container_client.get_blob_client(blob_name)
        try:
            blob_data = blob_client.download_blob()
            return blob_data.content_as_text()
        except ResourceNotFoundError:
            return None
    
    def download_raw_blob(self, container_name, blob_name):
        container_client = self.blob_service_client.get_container_client(container_name)
        blob_client = container_client.get_blob_client(blob_name)
        try:
            blob_data = blob_client.download_blob()
            return blob_data
        except ResourceNotFoundError:
            return None

    def delete_blob(self, container_name, blob_name):
        container_client = self.blob_service_client.get_container_client(container_name)
        blob_client = container_client.get_blob_client(blob_name)
        blob_client.delete_blob()



from Bio import Entrez
import json
import pkg_resources

class PubMedEntrez:
    def __init__(self):
        # load the connection string from a config file
        config = json.loads(pkg_resources.resource_string(__name__, "config.json"))

        self.email = config["pubmed"]["email"]

    # http://biopython.org/DIST/docs/tutorial/Tutorial.html#sec146
    def search(self, query, max_results=100, skip_results=0):
        Entrez.email = self.email
        handle = Entrez.esearch(db='pubmed',
                                retstart=skip_results,
                                sort='relevance',
                                retmax=max_results,
                                retmode='xml',
                                term=query)
        results = Entrez.read(handle)
        return results
    
    # http://biopython.org/DIST/docs/tutorial/Tutorial.html#sec%3Aefetch
    def fetch_details(self, id_list):
        ids = ','.join(id_list)
        Entrez.email = self.email
        handle = Entrez.efetch(db='pubmed',
                                retmode='xml',
                                id=ids)
        results = Entrez.read(handle)
        return results
    
    # http://biopython.org/DIST/docs/tutorial/Tutorial.html#sec147
    def post_ids(self, id_list):
        ids = ','.join(id_list)
        Entrez.email = self.email
        handle = Entrez.epost(db='pubmed',
                                id=ids)
        search_results = Entrez.read(handle)
        webenv = search_results["WebEnv"]
        query_key = search_results["QueryKey"]

        results = { "WebEnv": webenv, "QueryKey": query_key }
        return results
    
    # http://biopython.org/DIST/docs/tutorial/Tutorial.html#sec%3Aentrez-webenv
    def fetch_details_by_webenv(self, webenv, query_key, max_results=100, skip_results=0):
        Entrez.email = self.email
        handle = Entrez.efetch(db='pubmed',
                                retstart=skip_results,
                                retmax=max_results,
                                retmode='xml',
                                webenv=webenv,
                                query_key=query_key)
        results = Entrez.read(handle)
        return results
    
        # http://biopython.org/DIST/docs/tutorial/Tutorial.html#sec%3Aentrez-webenv
        # batch_size = 50
        # print("Count: " + str(itemsCount))
        # out_handle = open("pubmed.temp", "w")
        # for start in range(0, itemsCount, batch_size):
        #     end = min(itemsCount, start + batch_size)
        #     print("Going to download record %i to %i" % (start + 1, end))
        #     fetch_handle = Entrez.efetch(
        #         db="pubmed",
        #         retmode="xml",
        #         retstart=start,
        #         retmax=batch_size,
        #         webenv=webenv,
        #         query_key=query_key)
        #     data =  Entrez.read(fetch_handle)
        #     print(json.dumps(data))
        #     fetch_handle.close()
        #     out_handle.write(json.dumps(data))
        # out_handle.close()
        
        # return out_handle


from .pubmed_entrez import PubMedEntrez
from .azure_storage import AzureStorage
import json


class PubMedDownload:
    def __init__(self):
        self.pubmed_api = PubMedEntrez()
        self.azure_storage = AzureStorage()

    def download_to_azure(self, query, container_name, file_name, max_results=100, skip_results=0, batch_size = 50):
        blob = self.azure_storage.download_blob(container_name, file_name)
        
        if blob is None:
            blob_data = []
        else:
            blob_data = json.loads(blob.decode('utf-8'))
        
        search_results = self.pubmed_api.search(query, max_results, skip_results)
        id_list = search_results["IdList"]
        
        total_results = int(search_results["Count"])

        returned_results = len(id_list)
        print("Pubmed returned " + str(returned_results) + " results out of " + str(total_results) + " total results")

        # post the IDs to the server
        post_results = self.pubmed_api.post_ids(id_list)

        # Get the WebEnv and QueryKey
        webenv = post_results["WebEnv"]
        query_key = post_results["QueryKey"]

        for start in range(0, returned_results, batch_size):
            end = min(returned_results, start + batch_size)
            print("Going to download record %i to %i" % (start + 1, end))
            try:
                data = self.pubmed_api.fetch_details_by_webenv(webenv, query_key, batch_size, start)
                # print(data["PubmedArticle"])
                blob_data.extend(data["PubmedArticle"])
            except Exception as e:
                print("Error downloading records " + str(start + 1) + " to " + str(end))
                print(e)
                break

        # Upload the file to Azure Storage
        self.azure_storage.upload_blob(container_name, file_name, json.dumps(blob_data), True)

        print("Downloaded " + str(returned_results) + " records")



import os, sys
sys.path.append(os.path.join(os.path.dirname(__file__), ".."))


import pinecone
from tqdm import tqdm
from time import sleep
import json
from .helper import chunks, get_secret
import pkg_resources

class VectorDB:
    def __init__(self):
         # load the connection string from a config file
        config = json.loads(pkg_resources.resource_string(__name__, "config.json"))
        
        index_name = config["pinecone"]["index_name"]
        api_key = get_secret("pinecone-api-key")
        environment = config["pinecone"]["environment"]
        self.vector_size = config["pinecone"]["vector_size"]
               
        pinecone.init(
            api_key=api_key,
            environment=environment, 
            vector_size=self.vector_size, 
        )
       
        self.index = pinecone.Index(index_name)
        # view index stats
        print(self.index.describe_index_stats())

    

    def is_vector_indexed(self, id):
        vector1 = [float(0.1) for i in range(self.vector_size)]
        res = self.index.query(
            vector=vector1,
            filter={
                'id': id
            },
            top_k=1, 
            include_metadata=False
        )

        return len(res['matches']) == 1
    
    def upsert(self, ids, embeddings, metadata, namespace=None, batch_size=100):
        to_upsert = list(zip(ids, embeddings, metadata))
       
        for to_upsert_chunk in chunks(to_upsert, batch_size):
            self.index.upsert(
                vectors=to_upsert_chunk,
                namespace=namespace
            )
    
    def query(self, vector, filter=None, top_k=10, include_metadata=True, namespace=None):
        res = self.index.query(
            vector=vector,
            filter=filter,
            top_k=top_k, 
            include_metadata=include_metadata,
            namespace=namespace
        )
        return res
    


import itertools
import json
import pkg_resources

# A helper function to break an iterable into chunks of size batch_size.
def chunks(iterable, batch_size=100):        
    it = iter(iterable)
    chunk = tuple(itertools.islice(it, batch_size))
    while chunk:
        yield chunk
        chunk = tuple(itertools.islice(it, batch_size))


from azure.identity import DefaultAzureCredential
from azure.keyvault.secrets import SecretClient

def get_secret(secret_name):
    credential = DefaultAzureCredential(additionally_allowed_tenants=['*'])
    secret_client = SecretClient(vault_url="https://trialhub-helpers.vault.azure.net/", credential=credential)
    secret = secret_client.get_secret(secret_name)
    return secret.value

def get_pubmed_types():
    config = json.loads(pkg_resources.resource_string(__name__, "config.json"))
    return config["pubmed"]["relevant_publication_types"]


import os
from setuptools import setup, find_packages

def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname), "r", encoding="utf-8") as fh:
        return fh.read()
    
setup(
    name='trialhub',
    version='0.0.9',
    description='Helper modules for TrialHub project',
    author='FindMeCure team',
    author_email='ivo@findmecure.com',
    url='https://github.com/IvayloYosifov/trialhub-helpers',
    packages=['trialhub'],    
    package_data={'trialhub':['config.json']},
    install_requires=read("requirements.txt"),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
# py_modules=['vector_db', 'helper', 'azure_storage', 'pubmed_entrez', 'pubmed_download', 'open_ai'],


import pymongo
import json
from .helper import get_secret
import pkg_resources

class MongoDB:
    def __init__(self):
         # load the connection string from a config file
        config = json.loads(pkg_resources.resource_string(__name__, "config.json"))
        
        connection_string = get_secret("mongodb-connection-string")
        db_name = config["mongodb"]["db_name"]
        
        client = pymongo.MongoClient(connection_string, serverSelectionTimeoutMS=5000)
        try:
            print(client.server_info())
        except Exception:
            print("Unable to connect to the server.")

        self.db = client[db_name]
        